# Copyright (C) 2006-2008 Red Hat, Inc.
#
# This program is free software; you can redistribute
# it and/or modify it under the terms of version 2 of the
# GNU General Public License as published by the
# Free Software Foundation.

from conga_storage_constants import ADD_SOURCES, CLUNAME, CLUSTER_STORAGE, \
	CREATE_MAPPER, get_pretty_mapper_info, PAGETYPE, PT_MAPPER_ID, \
	PT_MAPPER_TYPE, PT_PATH, STONAME, STORAGE, STORAGESYS, \
	VIEW_BD, VIEW_BDS, VIEW_MAPPER, VIEW_MAPPERS

from ricci_defines import MAPPER_SYS_TYPE, MAPPER_VG_TYPE, \
	SYSTEM_PREFIX, VG_PREFIX

from LuciZope import get_systems_statuses

from StorageReport import invalidate_storage_report

def createStorageChooser(self, request, systems):
	dummynode = {}

	# First, see if a storage system is chosen, then
	# check that the current user can access that system
	stoname = request.get(STONAME) or ''
	url = request.get('URL') or '.'
	pagetype = request.get(PAGETYPE) or '0'

	try:
		if stoname and request.has_key('reprobe_storage'):
			invalidate_storage_report(request.SESSION, stoname)
			redirect_url = '%s?%s' % (request['ACTUAL_URL'], request['QUERY_STRING'])
			redirect_url = redirect_url.replace('&reprobe_storage=true', '')
			request.RESPONSE.redirect(redirect_url)
	except:
		pass

	sdata = {}
	sdata['Title'] = 'System List'
	sdata['cfg_type'] = 'storages'
	sdata['absolute_url'] = '%s?pagetype=%s' % (url, STORAGESYS)
	sdata['Description'] = 'Systems available for storage configuration'

	if pagetype == STORAGESYS or pagetype == '0':
		sdata['show_children'] = True
		sdata['currentItem'] = True
	else:
		sdata['currentItem'] = False
		sdata['show_children'] = False

	syslist = list()
	if sdata['show_children']:
		#display_clusters = True
		display_clusters = False
		if display_clusters:
			sorted_data = self.group_systems_by_cluster(systems, from_cache=False)
			for sdl in sorted_data[:2]:
				for data in sdl:
					createStorageChooser_inner(url, pagetype, stoname, data, syslist)
		else:
			sorted_data = get_systems_statuses(self, systems, from_cache=False)
			for data in sorted_data:
				createStorageChooser_inner(url, pagetype, stoname, data, syslist)
	sdata['children'] = syslist

	mylist = list()
	mylist.append(sdata)
	#mylist.append(sadd)
	#mylist.append(scfg)
	dummynode['children'] = mylist

	return dummynode

def createStorageChooser_inner(url, pagetype, stoname, system_data, syslist):
	ssys = {}
	if 'nodes' in system_data:
		title = system_data['name']
		if system_data['alias'] != '':
			title = '%s (%s)' % (system_data['alias'], title)
		ssys['Title'] = 'Cluster %s' % title
		ssys['cfg_type'] = 'storage'
		ssys['absolute_url'] = '%s?%s=%s&%s=%s' \
			% (url, PAGETYPE, CLUSTER_STORAGE, CLUNAME, system_data['name'])
		ssys['Description'] = 'Configure shared storage of cluster %s' % title
		ssys['currentItem'] = False
		ssys['show_children'] = True
		kids = []
		for node in system_data['nodes']:
			createStorageChooser_inner(url, pagetype, stoname, node, kids)
		ssys['children'] = kids
	else:
		if system_data['authed'] is False:
			return
		ssys['Title'] = system_data['hostname']
		ssys['cfg_type'] = 'storage'
		ssys['absolute_url'] = '%s?%s=%s&%s=%s' \
			% (url, PAGETYPE, STORAGE, STONAME, system_data['hostname'])
		ssys['Description'] = 'Configure storage on %s' % system_data['hostname']

		if pagetype == STORAGE:
			if stoname == system_data['hostname']:
				ssys['currentItem'] = True
			else:
				ssys['currentItem'] = False
		else:
			ssys['currentItem'] = False
	syslist.append(ssys)
	return

def create_mapper_subitem(storage_report, request, mapper_list, mapper_templ_list):
	pagetype_req = request.get(PAGETYPE) or '0'
	mapper_type_req = request.get(PT_MAPPER_TYPE) or ''
	mapper_id_req = request.get(PT_MAPPER_ID) or ''
	path_req = request.get(PT_PATH) or ''
	url = request.get('URL') or '.'
	hostname = request.get(STONAME) or ''

	mapper_type = None
	if len(mapper_list) != 0:
		mapper_type = mapper_list[0].getAttribute('mapper_type')
	if len(mapper_templ_list) != 0:
		mapper_type = mapper_templ_list[0].getAttribute('mapper_type')
	if mapper_type is None:
		return None

	buff, dummy1, dummy2 = get_pretty_mapper_info(mapper_type)
	pretty_names = '%ss' % buff
	pretty_names_desc = 'Manage %ss' % buff
	pretty_name_desc = 'Manage %s' % buff
	pretty_new_name = 'New %s' % buff
	pretty_new_name_desc = 'Create New %s' % buff

	srs_p = {}
	srs_p['Title'] = pretty_names
	srs_p['cfg_type'] = 'nodes'
	srs_p['absolute_url'] = '%s?%s=%s&%s=%s&%s=%s' \
		% (url, PAGETYPE, VIEW_MAPPERS, STONAME, hostname, PT_MAPPER_TYPE, mapper_type)
	srs_p['Description'] = pretty_names_desc
	if pagetype_req in ( VIEW_MAPPERS, VIEW_MAPPER, ADD_SOURCES, CREATE_MAPPER, VIEW_BD ) and mapper_type_req == mapper_type:
		srs_p['show_children'] = True
	else:
		srs_p['show_children'] = False

	if pagetype_req == VIEW_MAPPERS and mapper_type_req == mapper_type:
		srs_p['currentItem'] = True
	else:
		srs_p['currentItem'] = False

	srs_list = []

	# new mapper
	if len(mapper_templ_list) != 0:
		sr = {}
		sr['Title'] = pretty_new_name
		sr['cfg_type'] = 'nodes'
		sr['absolute_url'] = '%s?%s=%s&%s=%s&%s=%s' \
			% (url, PAGETYPE, CREATE_MAPPER, STONAME, hostname, PT_MAPPER_TYPE, mapper_type)
		sr['Description'] = pretty_new_name_desc
		sr['show_children'] = False

		if pagetype_req == CREATE_MAPPER and mapper_type_req == mapper_type:
			sr['currentItem'] = True
		else:
			sr['currentItem'] = False

		srs_list.append(sr)

	# existing mappers
	for sr_xml in mapper_list:
		sr_id = sr_xml.getAttribute('mapper_id')
		srname = sr_id.replace('%s:' % mapper_type, '').replace('/dev/', '')

		if srname == '' and mapper_type == MAPPER_VG_TYPE and sr_id == VG_PREFIX:
			#srname = 'Uninitialized PVs'
			continue

		sr = {}
		sr['Title'] = srname
		sr['cfg_type'] = 'nodes'
		sr['absolute_url'] = '%s?%s=%s&%s=%s&%s=%s&%s=%s' \
			% (url, PAGETYPE, VIEW_MAPPER, STONAME, hostname, PT_MAPPER_TYPE, mapper_type, PT_MAPPER_ID, sr_id)
		sr['Description'] = pretty_name_desc

		if pagetype_req in ( VIEW_MAPPER, ADD_SOURCES, VIEW_BD ) and mapper_id_req == sr_id:
			sr['currentItem'] = True
		else:
			sr['currentItem'] = False
		sr['show_children'] = False

		# targets
		tgts_list = []
		for tgt_xml in storage_report.get_targets(sr_xml):
			tg_path = tgt_xml.getAttribute('path')
			tgname = tg_path
			#if tg_path != path_req:
			#	continue
			tg = {}
			tg['Title'] = tgname
			tg['cfg_type'] = 'nodes'
			tg['absolute_url'] = '%s?%s=%s&%s=%s&%s=%s&%s=%s&%s=%s' \
				% (url, PAGETYPE, VIEW_BD, STONAME, hostname, PT_MAPPER_TYPE, mapper_type, PT_MAPPER_ID, sr_id, PT_PATH, tg_path)
			tg['Description'] = tgname
			tg['show_children'] = False

			if pagetype_req == VIEW_BD and path_req == tg_path:
				tg['currentItem'] = True
			else:
				tg['currentItem'] = False
			tgts_list.append(tg)

		sr['children'] = tgts_list
		srs_list.append(sr)

	if len(srs_list) != 0:
		srs_p['children'] = srs_list
		return srs_p

	return None

def createStorageConfigTree(self, request, storage_report):
	dummynode = {}
	if storage_report is None:
		return dummynode

	pagetype = request.get(PAGETYPE) or '0'
	mapper_type = request.get(PT_MAPPER_TYPE) or ''
	mapper_id = request.get(PT_MAPPER_ID) or ''
	path = request.get(PT_PATH) or ''
	url = request.get('URL') or '.'
	hostname = request.get(STONAME) or ''

	### Hard Drives ###

	hds_pretty_name, hd_pretty_name, dummy = get_pretty_mapper_info(MAPPER_SYS_TYPE)

	hds_p = {}
	hds_p['Title'] = hds_pretty_name
	hds_p['cfg_type'] = 'nodes'
	hds_p['absolute_url'] = '%s?%s=%s&%s=%s&%s=%s&%s=%s' \
		% (url, PAGETYPE, VIEW_BDS, STONAME, hostname, PT_MAPPER_TYPE, MAPPER_SYS_TYPE, PT_MAPPER_ID, SYSTEM_PREFIX)
	hds_p['Description'] = 'Manage %s' % hds_pretty_name

	if (pagetype == VIEW_BDS or pagetype == VIEW_BD) and mapper_type == MAPPER_SYS_TYPE:
		hds_p['show_children'] = True
		if pagetype == VIEW_BDS:
			hds_p['currentItem'] = True
		else:
			hds_p['currentItem'] = False
	else:
		hds_p['show_children'] = False

	targets_list = storage_report.get_targets(storage_report.get_mapper(SYSTEM_PREFIX))

	hds_list = []
	for target in targets_list:
		sys_id = target.getAttribute('mapper_id')
		hd_path = target.getAttribute('path')

		hd = {}
		hd['Title'] = hd_path.replace('/dev/', '')
		hd['cfg_type'] = 'nodes'
		hd['absolute_url'] = '%s?%s=%s&%s=%s&%s=%s&%s=%s&%s=%s' \
			% (url, PAGETYPE, VIEW_BD, STONAME, hostname, PT_MAPPER_TYPE, MAPPER_SYS_TYPE, PT_MAPPER_ID, sys_id, PT_PATH, hd_path)
		hd['Description'] = 'Manage %s' % hd_pretty_name
		hd['show_children'] = False

		if pagetype == VIEW_BD and mapper_id == sys_id and path == hd_path:
			hd['currentItem'] = True
		else:
			hd['currentItem'] = False

		hds_list.append(hd)

	hds_p['children'] = hds_list

	## mappers ##

	main_kids = [ hds_p ]

	mappers_dir = storage_report.get_mappers_dir()
	mapper_templs_dir = storage_report.get_mapper_temps_dir()

	glo_dir = {}
	for cur_type in mappers_dir:
		glo_dir[cur_type] = [ mappers_dir[cur_type], [] ]

	for cur_type in mapper_templs_dir:
		if cur_type not in glo_dir:
			glo_dir[cur_type] = [ [], mapper_templs_dir[cur_type] ]
		else:
			glo_dir[cur_type][1] = mapper_templs_dir[cur_type]

	for cur_type in glo_dir:
		if cur_type == MAPPER_SYS_TYPE:
			continue
		item = create_mapper_subitem(storage_report, request, glo_dir[cur_type][0], glo_dir[cur_type][1])
		if item is None:
			continue
		else:
			main_kids.append(item)

	dummynode['children'] = main_kids
	return dummynode

def getStorageURL(self, request, hostname):
	# return URL to manage this storage system
	baseurl = request.get('URL') or '.'
	return '%s?%s=%s&%s=%s' % (baseurl, PAGETYPE, str(STORAGE), STONAME, hostname)
